      subroutine calcpar_nests(n,uuhn,vvhn)
C                              i  i    i    o
********************************************************************************
* $Id: calcpar_nests.f,v 1.1 2006/08/10 13:52:02 paul Exp paul $
* $Log: calcpar_nests.f,v $
* Revision 1.1  2006/08/10 13:52:02  paul
* Initial revision
*
*                                                                              *
*     Computation of several boundary layer parameters needed for the          *
*     dispersion calculation and calculation of dry deposition velocities.     *
*     All parameters are calculated over the entire grid.                      *
*     This routine is similar to calcpar, but is used for the nested grids.    *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     8 February 1999                                                          *
*                                                                              *
* ------------------------------------------------------------------           *
*     Petra Seibert, Feb 2000:                                                 *
*     convection scheme:                                                       *
*     new variables in call to richardson                                      *
*                                                                              *
********************************************************************************
*  Changes, Bernd C. Krueger, Feb. 2001:
*        Variables tth and qvh (on eta coordinates) in common block
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* n                  temporal index for meteorological fields (1 to 3)         *
*                                                                              *
* Constants:                                                                   *
*                                                                              *
*                                                                              *
* Functions:                                                                   *
* scalev             computation of ustar                                      *
* obukhov            computation of Obukhov length                              *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer n,ix,jy,i,l,kz,lz,kzmin
      real ttlev(nuvzmax),qvlev(nuvzmax),obukhov,scalev,ol,hmixplus
      real ulev(nuvzmax),vlev(nuvzmax),ew,rh,vd(maxspec),subsceff,ylat
      real altmin,tvold,pold,zold,pint,tv,zlev(nuvzmax),const
      real uuhn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests)
      real vvhn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests)
!      real pvhn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests)
      real xlon,tkelev(nuvzmax),rhoa,rhow,e,sigmatoh,wm,thetaref
!      real tkehn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests)

C Loop over all nests
*********************

      do 80 l=1,numbnests

C Set nest proj specs.

         call map_set(code_sav,lat1n_sav(l),lon1n_sav(l),ki_sav(l),
     &kj_sav(l),dxn_sav(l),stdlon_sav,truelat1_sav,truelat2_sav)

C Loop over entire grid
***********************

      do 10 jy=0,nyn(l)-1
         do 10 ix=0,nxn(l)-1
C Set minimum height for tropopause
***********************************
        call ij_to_latlon(float(ix),float(jy),ylat,xlon)

        if ((ylat.ge.-20.).and.(ylat.le.20.)) then
          altmin = 5000.
        else
          if ((ylat.gt.20.).and.(ylat.lt.40.)) then
            altmin=2500.+(40.-ylat)*125.
          else if ((ylat.gt.-40.).and.(ylat.lt.-20.)) then
            altmin=2500.+(40.+ylat)*125.
          else
            altmin=2500.
          endif
        endif

C 1) Calculation of friction velocity
*************************************
C Direct model output 
C          ustarn(ix,jy,1,n,l)=scalev(psn(ix,jy,1,n,l),tt2n(ix,jy,1,n,l),
C     +    td2n(ix,jy,1,n,l),surfstrn(ix,jy,1,n,l))

C 2) Calculation of inverse Obukhov length scale
************************************************

          ol=obukhov(psn(ix,jy,1,n,l),tthn(ix,jy,1,n,l),
     +    qvhn(ix,jy,1,n,l),ustarn(ix,jy,1,n,l),sshfn(ix,jy,1,n,l))
          if (ol.ne.0.) then
            olin(ix,jy,1,n,l)=1./ol
          else
            olin(ix,jy,1,n,l)=99999.
          endif


C 3) Calculation of convective velocity scale and mixing height
***************************************************************

          do 20 i=1,nuvz
            ulev(i)=uuhn(ix,jy,i,l)
            vlev(i)=vvhn(ix,jy,i,l)
           qvlev(i)=qvh(ix,jy,i,n)
            ttlev(i)=tthn(ix,jy,i,n,l)
20          tkelev(i)=tkehn(ix,jy,i,l)

          call richardson(psn(ix,jy,1,n,l),ustarn(ix,jy,1,n,l),ttlev,
     +    qvlev,ulev,vlev,nuvz,sigmahalf,ptop,p00,ts0,ta,
     +    sshfn(ix,jy,1,n,l),hmixn(ix,jy,1,n,l),wstarn(ix,jy,1,n,l),
     +    hmixplus,ix,jy)

cc        if (pblscaling) then
*     
*     XMIX PREDICTED QUANTITY: CALCULATE XMIXPLUS ONLY
*     
           if(lsubgrid.eq.1) then
              call calchmixplus(psn(ix,jy,1,n,l),ttlev,ulev,vlev,
     +             nuvz,sigmahalf,ptop,p00,ts0,ta,
     +             hmixn(ix,jy,1,n,l),hmixplus)
           endif
cc        else
*
*     TKE PREDICTED QUANTITY: CALCULATE HMIX FROM TKE PROFILE,
*     CALCULATE HMIXPLUS
*     
cc           call tkeprofile(psn(ix,jy,1,n,l),ttlev,ulev,vlev,tkelev,
cc     +          nuvz,sigmahalf,ptop,p00,ts0,ta,
cc     +          hmixn(ix,jy,1,n,l),hmixplus)
           
           wm=qvhn(ix,jy,1,n,l)
           tv=tthn(ix,jy,1,n,l)*(1.+.378*wm/(wm+.622))
           thetaref=tv*(100000./psn(ix,jy,1,n,l))**kappa
           if(sshfn(ix,jy,1,n,l).lt.0.0) then
              wstarn(ix,jy,1,n,l)=(-hmixn(ix,jy,1,n,l)*ga/thetaref*
     &             sshfn(ix,jy,1,n,l)/cpa)**0.333
           else
              wstarn(ix,jy,1,n,l)=0.0
           endif
cc        endif

          if(lsubgrid.eq.1) then
            subsceff=min(excessoron(ix,jy,l),hmixplus)
          else
            subsceff=0
          endif
*
* CALCULATE HMIX EXCESS ACCORDING TO SUBGRIDSCALE VARIABILITY AND STABILITY
*
          hmixn(ix,jy,1,n,l)=hmixn(ix,jy,1,n,l)+subsceff
          hmixn(ix,jy,1,n,l)=max(hmixmin,hmixn(ix,jy,1,n,l)) ! minim PBL height
          hmixn(ix,jy,1,n,l)=min(hmixmax,hmixn(ix,jy,1,n,l)) ! maxim PBL height


C 4) Calculation of dry deposition velocities
*********************************************

          if (DRYDEP) then
            z0(4)=0.016*ustarn(ix,jy,1,n,l)*ustarn(ix,jy,1,n,l)/ga
            z0(9)=0.016*ustarn(ix,jy,1,n,l)*ustarn(ix,jy,1,n,l)/ga

C Calculate relative humidity at surface
****************************************
C            rh=ew(td2n(ix,jy,1,n,l))/ew(tt2n(ix,jy,1,n,l))
            rhoa=psn(ix,jy,1,n,l)/(r_air*tth(ix,jy,1,n)) ! Air density
            rhow=qvhn(ix,jy,1,n,l)*rhoa                 ! Water Vapor density
            e=rhow*r_h2o*tthn(ix,jy,1,n,l)              ! Water Vapor Pressure
            rh=e/ew(tthn(ix,jy,1,n,l))

            call getvdep(n,ix,jy,ustarn(ix,jy,1,n,l),
     +      tthn(ix,jy,1,n,l),psn(ix,jy,1,n,l),1./olin(ix,jy,1,n,l),
     +      ssrn(ix,jy,1,n,l),rh,lsprecn(ix,jy,1,n,l)+
     +      convprecn(ix,jy,1,n,l),vd)

            do 30 i=1,nspec
30            vdepn(ix,jy,i,n,l)=vd(i)

          endif

*******************************************************
C Calculate height of thermal tropopause (Hoinka, 1997)
*******************************************************

C 1) Calculate altitudes of ECMWF model levels
**********************************************

C          tvold=tt2n(ix,jy,1,n,l)*(1.+0.378*ew(td2n(ix,jy,1,n,l))/
C     +                                   psn(ix,jy,1,n,l))
CC          pold=psn(ix,jy,1,n,l)
          zold=0.
          do 40 kz=2,nuvz
             zlev(kz)=zold+sigmatoh(sigmahalf(kz),psn(ix,jy,1,n,l),ptop
     +       ,p00,ts0,ta)
C            pint=akz(kz)+bkz(kz)*psn(ix,jy,1,n,l)  ! pressure on model layers
C            tv=tthn(ix,jy,kz,n,l)*(1.+0.608*qvhn(ix,jy,kz,n,l))

C            if (abs(tv-tvold).gt.0.2) then
C             zlev(kz)=zold+const*log(pold/pint)*(tv-tvold)/log(tv/tvold)
C            else
C              zlev(kz)=zold+const*log(pold/pint)*tv
C            endif
CC            tvold=tv
CC            pold=pint
            zold=zlev(kz)
40          continue

C 2) Define a minimum level kzmin, from which upward the tropopause is
C    searched for. This is to avoid inversions in the lower troposphere
C    to be identified as the tropopause
*************************************************************************

          do 44 kz=1,nuvz
            if (zlev(kz).ge.altmin) then
              kzmin=kz
              goto 45
            endif
44          continue
45        continue

C 3) Search for first stable layer above minimum height that fulfills the
C    thermal tropopause criterion
*************************************************************************

          do 50 kz=kzmin,nuvz
            do 60 lz=kz+1,nuvz
              if ((zlev(lz)-zlev(kz)).gt.2000.) then
                if (((tthn(ix,jy,kz,n,l)-tthn(ix,jy,lz,n,l))/
     +          (zlev(lz)-zlev(kz))).lt.0.002) then
                  tropopausen(ix,jy,1,n,l)=zlev(kz)
                  goto 51
                endif
                goto 50
              endif
60            continue
50          continue
51        continue


10        continue


C Calculation of potential vorticity on 3-d grid, if plume trajectory mode is used
**********************************************************************************
CPSK FIXXME
C
C
        if ((iout.eq.4).or.(iout.eq.5)) then
C          call calcpv_nests(l,n,uuhn,vvhn,pvhn)
	  stop 'Sorry plume trajectories not yet included !'	
        endif

80      continue

! Set map back to mother
        call map_set(code_sav,lat1_sav,lon1_sav,1.,1.,dx_sav,stdlon_sav
     &       ,truelat1_sav,truelat2_sav)

      end
