      real function cloudwater(ps,ttprof,clwprof,nz,sigmahalf,
C                              i    i       i    i      i
     &   sigmafull,ptop,p00,ts0,ta)
C            i      i    i   i  i
************************************************************************
* $Id: cloudwater.f,v 1.1 2006/08/09 11:35:35 paul Exp paul $
* $Log: cloudwater.f,v $
* Revision 1.1  2006/08/09 11:35:35  paul
* Initial revision
*
*                                                                      *
*                FLEXPART (MM5) SUBROUTINE CLOUDWATER                  *
*                          Author: G. WOTAWA                           *
*                              1998-08-11                              *
*                                                                      *
************************************************************************
*                                                                      *
* SHORT DESCRIPTION:                                                   *
*                                                                      *
* This subroutine calculates the total integrated liquid water content *
* of the atmospheric column (kg/m2) for subsequent use in the wet      *
* deposition part of FLEXPART.                                         *
*                                                                      *
************************************************************************
*                                                                      *
* DESCRIPTION OF INPUT:                                                *
*                                                                      *
* PS              (REFERENCE) SURFACE PRESSURE (PA)                    *
* TTPRO           VERTICAL PROFILE OF TEMPERATURE (K)                  *
* CLWPROF         VERTICAL PROFILE OF CLOUD WATER (KG/KG)              *
* NZ              NUMBER OF HALF SIGMA LEVELS                          *
* SIGMAHALF       HALF SIGMA LEVELS MODEL                              *
* SIGMAFULL       FULL SIGMA LEVELS MODEL                              *
* PTOP            MODEL TOP PRESSURE (PA)                              *
* P00             REFERENCE SEA LEVEL PRESSURE (PA)                    *
* TS0             REFEENCE SEA LEVEL TEMPERATURE (K)                   *
* TA              REFERENCE TEMPERATURE LAPSE RATE (K)                 *
* ORO             TERRAIN HEIGHT (M)                                   *
*                                                                      *
************************************************************************
      include 'includepar'

      integer nz,kz
      real ps,ttprof(nz),clwprof(nz),sigmahalf(nz)
      real sigmafull(nz+1),ptop,pint,rhoair,deltah,z1,z2
      real p00,ts0,ta,sigmatoh,geopot1

      geopot1=sigmatoh(sigmafull(1),ps,ptop,p00,ts0,ta)

      cloudwater=0.0
      do 10 kz=1,nz
        pint=sigmahalf(kz)*(ps-ptop)+ptop
        rhoair=pint/(r_air*ttprof(kz))
        z1=sigmatoh(sigmafull(kz),  ps,ptop,p00,ts0,ta)-geopot1
        z2=sigmatoh(sigmafull(kz+1),ps,ptop,p00,ts0,ta)-geopot1
        deltah=z2-z1
        cloudwater=cloudwater+clwprof(kz)*rhoair*deltah
10    continue

      return
      end
