      subroutine gridcheck()
***********************************************************************
* $Id: gridcheck.f,v 1.1 2006/06/20 13:46:21 paul Exp paul $
* $Log: gridcheck.f,v $
* Revision 1.1  2006/06/20 13:46:21  paul
* Initial revision
*
*                                                                     * 
*             TRAJECTORY MODEL SUBROUTINE GRIDCHECK                   *
*             FLEXPART VERSION -> DO NOT USE IN FLEXTRA, PRAEPRO      *
*                                                                     *
***********************************************************************
*                                                                     * 
*             AUTHOR:      G. WOTAWA                                  *
*             DATE:        1997-08-06                                 *
*             LAST UPDATE: 1997-10-10                                 *
*                                                                     * 
*             Update:      1999-02-08, global fields allowed, A. Stohl* 
*                                                                     * 
***********************************************************************
*                                                                     *
* DESCRIPTION:                                                        *
*                                                                     *
* THIS SUBROUTINE DETERMINES THE GRID SPECIFICATIONS (LOWER LEFT      *
* LONGITUDE, LOWER LEFT LATITUDE, NUMBER OF GRID POINTS, GRID DIST-   *
* ANCE AND VERTICAL DISCRETIZATION OF THE ECMWF MODEL) FROM THE       *
* GRIB HEADER OF THE FIRST INPUT FILE. THE CONSISTANCY (NO CHANGES    *
* WITHIN ONE FLEXPART RUN) IS CHECKED IN THE ROUTINE "READWIND" AT    *
* ANY CALL.                                                           *
*                                                                     *
* XLON0                geographical longitude of lower left gridpoint *
* YLAT0                geographical latitude of lower left gridpoint  *
* NX                   number of grid points x-direction              *
* NY                   number of grid points y-direction              *
* DX                   grid distance x-direction                      *
* DY                   grid distance y-direction                      *
* NUVZ                 number of grid points for horizontal wind      *
*                      components in z direction                      *
* NWZ                  number of grid points for vertical wind        *
*                      component in z direction                       *
* sizesouth, sizenorth give the map scale (i.e. number of virtual grid*
*                      points of the polar stereographic grid):       *
*                      used to check the CFL criterion                *
* SIGMAHALF(1)-        heights of gridpoints where u and v are        *
* SIGMAHALF(NUVZ)      given                                          *
* SIGMAFULL(1)-        heights of gridpoints where w is given         *
* SIGMAFULL(NWZ)                                                      *
*                                                                     *
***********************************************************************
*
      include 'includepar'
      include 'includecom'
      include 'includeconv'

      integer ix,jy,i,ifn,ifield,j,k,iumax,iwmax,numskip
      real xaux1,xaux2,yaux1,yaux2,sizesouth,sizenorth,xauxa,pint

* VARIABLES AND ARRAYS NEEDED FOR GRIB DECODING

C dimension of isec2 at least (22+n), where n is the number of parallels or
C meridians in a quasi-regular (reduced) Gaussian or lat/long grid

C dimension of zsec2 at least (10+nn), where nn is the number of vertical
C coordinate parameters

      integer isec0(2),isec1(56),isec2(22+nxmax+nymax),isec3(2)
      integer isec4(64),inbuff(jpack),ilen,iswap,ierr,iword,lunit
      real zsec2(60+2*nuvzmax),zsec3(2),zsec4(jpunp)
      character*1 yoper,opt
      data yoper/'D'/
      real xr,yr
      iumax=0
      iwmax=0

      if(ideltas.gt.0) then
        ifn=1
      else
        ifn=numbwf
      endif
*
* OPENING OF DATA FILE (GRIB CODE)
*
5     call pbopen(lunit,path(3)(1:len(3))//wfname(ifn),'r',ierr)
      if(ierr.lt.0) goto 999

      ifield=0   
10    ifield=ifield+1

* GET NEXT FIELDS
*
      call pbgrib(lunit,inbuff,jpack,ilen,ierr)
      if(ierr.eq.-1) goto 30    ! EOF DETECTED
      if(ierr.lt.-1) goto 999   ! ERROR DETECTED
 
      ierr=1

C Check whether we are on a little endian or on a big endian computer
*********************************************************************

c     if (inbuff(1).eq.1112101447) then         ! little endian, swap bytes
c       iswap=1+ilen/4
c       call swap32(inbuff,iswap)
c     else if (inbuff(1).ne.1196575042) then    ! big endian
c       stop 'subroutine gridcheck: corrupt GRIB data'
c     endif

      call gribex(isec0,isec1,isec2,zsec2,isec3,zsec3,isec4,
     &            zsec4,jpunp,inbuff,jpack,iword,yoper,ierr)
      if (ierr.ne.0) goto 999   ! ERROR DETECTED

      if(ifield.eq.1) then
        nxfield=isec2(2)
        ny=isec2(3)
CPSK
C READ IN GRID SPECIFICATIONS
        open(unitwind,file=path(3)(1:len(3))//'GRID.TXT',
     &       status='old',err=996)
!     
        read(unitwind,*) codep
        if(codep.ne.1) goto 995
        codep=3                  !In Terrain LC has code 1, but map_utils
!     uses 3 for LC
        read(unitwind,*) dum_nx
        if(dum_nx.ne.nxfield) goto 996
        read(unitwind,*) dum_ny
        if(dum_ny.ne.ny)goto 997
        read(unitwind,*) nlev_ec
        read(unitwind,*) dxp
        dxp=dxp*1000.   ! Reference unit = m
        dyp=dxp                   ! Only dx=dy possible in MM5...
        read(unitwind,*) dum_nx_mother
        read(unitwind,*) dum_ny_mother
        read(unitwind,*) dum_dx_mother
        dum_dx_mother=dum_dx_mother*1000.
        dum_dy_mother=dum_dx_mother
        read(unitwind,*) knownip
        read(unitwind,*) knownjp
        read(unitwind,*) stdlonp
        read(unitwind,*) 
        read(unitwind,*) lon1p
        read(unitwind,*) lat1p
        read(unitwind,*)
        read(unitwind,*)
        read(unitwind,*) truelat1p
        read(unitwind,*) truelat2p
        read(unitwind,*) 
        read(unitwind,*) p00
        read(unitwind,*) ptop
        read(unitwind,*) ts0
        read(unitwind,*) ta
        do 15 k=1,nlev_ec
 15       read(unitwind,*) sigmahalf(nlev_ec-k+1)
        sigmafull(1)=1.0
        do 16 k=1,nlev_ec
 16       sigmafull(k+1)=2.*sigmahalf(k)-sigmafull(k)
        sigmafull(nlev_ec+1)=0.0
        write(*,*)
        write(*,'(a)') 'LEV    SIGMA   HALFSIGMA'
        write(*,*)
        do 17 k=1,nlev_ec
 17       write(*,'(i3,2f9.4)') k,sigmafull(k),sigmahalf(k)
        write(*,'(i3,f9.4)') nlev_ec+1,sigmafull(nlev_ec+1)
        write(*,*)

C     Compute extensions of domain
        call map_set(codep,lat1p,lon1p,knownip,knownjp,dxp,stdlonp,
     &truelat1p,truelat2p)  
C Save Motherdomain information
         code_sav=proj_code
         lat1_sav=proj_lat1
         lon1_sav=proj_lon1
  !       lat1p_sav=lat1p
  !       lon1p_sav=lon1p
         dx_sav=proj_dx
         stdlon_sav=proj_stdlon
         truelat1_sav=proj_truelat1
         truelat2_sav=proj_truelat2

CPS         call ij_to_latlon(1.,1.,yaux1,xaux1)
CPS         call ij_to_latlon(float(nxfield),float(ny),yaux2,xaux2)
         call ij_to_latlon(0.,0.,yaux1,xaux1)
         call ij_to_latlon(float(nxfield-1),float(ny-1),yaux2,xaux2)
         
!        xaux1=float(isec2(5))/1000. !Lon of 1st
!        xaux2=float(isec2(8))/1000. !Lon of last
        if (xaux1.gt.180) xaux1=xaux1-360.0
        if (xaux2.gt.180) xaux2=xaux2-360.0
        if (xaux1.lt.-180) xaux1=xaux1+360.0
        if (xaux2.lt.-180) xaux2=xaux2+360.0
        if (xaux2.lt.xaux1) xaux2=xaux2+360.
!        yaux1=float(isec2(7))/1000. !Lat of last
!        yaux2=float(isec2(4))/1000. !Lat of 1st
        xlon0=xaux1
        ylat0=yaux1
        dx=(xaux2-xaux1)/float(nxfield) 
        dy=(yaux2-yaux1)/float(ny)
!       dxconst=180./(dx*r_earth*pi)
!       dyconst=180./(dy*r_earth*pi)
!        print*,dxconst,dyconst,1./dx,1./dy,dx,dy,ylat0

!        do 991 j=1,ny
!           do 991 i=1,nxfield
!          call ij_to_latlon(float(i),float(j),yr,xr)        
!          inlon(i-1,j-1)=xr
!          inlat(i-1,j-1)=yr
!991     continue
!        dxconst=-999.
!        dyconst=-999.
!        do 992 j=1,ny-1
!           do 992 i=1,nxfield-1
!              call ij_to_latlon(float(i),float(j),yaux1,xaux1)
!              call ij_to_latlon(float(i+1),float(j+1),yaux2,xaux2)
!              dxconst(i-1,j-1)=180./((xaux2-xaux1)*r_earth*pi) 
!              dyconst(i-1,j-1)=180./((yaux2-yaux1)*r_earth*pi)
!              print*, dxconst(i-1,j-1), dyconst(i-1,j-1),i,j
! 992       continue       
!           print*,minval(dxconst),maxval(dxconst)
!           print*,minval(dyconst),maxval(dyconst)
!           print*,minloc(dxconst),maxloc(dxconst)
!           print*,minloc(dyconst),maxloc(dyconst) 
!           print*,nxfield,ny,nxmax,nymax
!           pause
!	print*,dxconst(44,77,0)
!	stop
	dxconst=1./dx_sav
	dyconst=1./dx_sav
!        nlev_ec=isec2(12)/2-1

C Check whether fields are global
C If they contain the poles, specify polar stereographic map
C projections using the stlmbr- and stcm2p-calls
************************************************************

        xauxa=abs(xaux2+dx-360.-xaux1)
        if (xauxa.lt.0.001) then
          nx=nxfield+1                 ! field is cyclic
          xglobal=.true.
          if (abs(nxshift).ge.nx)
     +    stop 'nxshift in file includepar is too large'
          xlon0=xlon0+float(nxshift)*dx
        else
          nx=nxfield
          xglobal=.false.
          if (nxshift.ne.0)
     +    stop 'nxshift (includepar) must be zero for non-global domain'
        endif
        nxmin1=nx-1
        nymin1=ny-1
        if (xlon0.gt.180.) xlon0=xlon0-360.
        xauxa=abs(yaux1+90.)
        if (xglobal.and.xauxa.lt.0.001) then
          sglobal=.true.               ! field contains south pole
C Enhance the map scale by factor 3 (*2=6) compared to north-south
C map scale
          sizesouth=6.*(switchsouth+90.)/dy
          call stlmbr(southpolemap,-90.,0.)
          call stcm2p(southpolemap,0.,0.,switchsouth,0.,sizesouth,
     +    sizesouth,switchsouth,180.)
          switchsouthg=(switchsouth-ylat0)/dy
        else
          sglobal=.false.
          switchsouthg=999999.
        endif
        xauxa=abs(yaux2-90.)
        if (xglobal.and.xauxa.lt.0.001) then
          nglobal=.true.               ! field contains north pole
C Enhance the map scale by factor 3 (*2=6) compared to north-south
C map scale
          sizenorth=6.*(90.-switchnorth)/dy
          call stlmbr(northpolemap,90.,0.)
          call stcm2p(northpolemap,0.,0.,switchnorth,0.,sizenorth,
     +    sizenorth,switchnorth,180.)
          switchnorthg=(switchnorth-ylat0)/dy
        else
          nglobal=.false.
          switchnorthg=999999.
        endif
      endif

      if (nxshift.lt.0) stop 'nxshift (includepar) must not be negative'
      if (nxshift.ge.nxfield) stop 'nxshift (includepar) too large'

      k=isec1(8)
      if(isec1(6).eq.131) iumax=max(iumax,nlev_ec-k+1)
      if(isec1(6).eq.135) iwmax=max(iwmax,nlev_ec-k+2)
  
      if(isec1(6).eq.129) then
        do 20 jy=0,ny-1
          do 20 ix=0,nxfield-1
            oro(ix,jy)=zsec4(nxfield*(ny-jy-1)+ix+1)
	    if (oro(ix,jy) .lt.0.) oro(ix,jy)=0.
20      continue
      endif
      if(isec1(6).eq.172) then
        do 25 jy=0,ny-1
          do 25 ix=0,nxfield-1
             if(int(zsec4(nx*(ny-jy-1)+ix+1)).eq.16) then
                lsm(ix,jy)=0.
             else
                lsm(ix,jy)=1.
             endif
25        continue
      endif
      if(isec1(6).eq.160) then
        do 27 jy=0,ny-1
          do 28 ix=0,nxfield-1
28          excessoro(ix,jy)=zsec4(nxfield*(ny-jy-1)+ix+1)
27        continue
      endif

      goto 10                      !! READ NEXT LEVEL OR PARAMETER

*
* CLOSING OF INPUT DATA FILE
*
30    call pbclose(lunit,ierr)     !! FINISHED READING / CLOSING GRIB FILE

      nuvz=iumax
      nwz =iwmax
      if(nuvz.eq.nlev_ec) nwz=nlev_ec+1

      if (nx.gt.nxmax) then                         
       write(*,*) 'FLEXPART error: Too many grid points in x direction.'
        write(*,*) 'Reduce resolution of wind fields.'
        write(*,*) 'Or change parameter settings in file includepar.'
        write(*,*) nx,nxmax
        stop
      endif

      if (ny.gt.nymax) then                         
       write(*,*) 'FLEXPART error: Too many grid points in y direction.'
        write(*,*) 'Reduce resolution of wind fields.'
        write(*,*) 'Or change parameter settings in file includepar.'
        write(*,*) ny,nymax
        stop
      endif

      if (nuvz+1.gt.nuvzmax) then                         
        write(*,*) 'FLEXPART error: Too many u,v grid points in z '//
     +  'direction.'
        write(*,*) 'Reduce resolution of wind fields.'
        write(*,*) 'Or change parameter settings in file includepar.'
        write(*,*) nuvz+1,nuvzmax
        stop
      endif

      if (nwz.gt.nwzmax) then                         
        write(*,*) 'FLEXPART error: Too many w grid points in z '//
     +  'direction.'
        write(*,*) 'Reduce resolution of wind fields.'
        write(*,*) 'Or change parameter settings in file includepar.'
        write(*,*) nwz,nwzmax
        stop
      endif

C If desired, shift all grids by nxshift grid cells
***************************************************

      if (xglobal) then
        call shift_field_0(oro,nxfield,ny)
        call shift_field_0(lsm,nxfield,ny)
        call shift_field_0(excessoro,nxfield,ny)
      endif

C Output of grid info
*********************

      write(*,*)
      write(*,*)
      write(*,'(a,2i7)') '# of vertical levels in MM5 data: ',
     +nuvz+1,nwz
      write(*,*)
      write(*,'(a)') 'Mother domain:'
      write(*,'(a,f10.1,a1,f10.1,a,f10.4,a,f10.4)') 
     +'  Longitude range: ',
     +xlon0,' to ',xlon0+(nxfield-1)*dx,'   Grid distance (ca. deg/meter
     +): ',
     +dx,'/',dxp
      write(*,'(a,f10.1,a1,f10.1,a,f10.4,a,f10.4)') 
     +'  Latitude range:  ',
     +ylat0,' to ',ylat0+(ny-1)*dy,'   Grid distance (ca. deg/meter): ',
     +dy,'/',dyp
      write(*,*)

      if (pblscaling) then
          print '(" *** Warning: BUG in CALCMIXPLUS           *** ")'
          print '(" *** HMIX SCALING ABL SCHEME NOT AVAILABLE *** ")'
	  print '(" *** using TKE SCALING ABL INSTEAD.........*** ",1/)'
          pblscaling=.false.
C         write(*,'(a,1/)') 'MM5: HMIX SCALING ABL SCHEME USED'
      else
         write(*,'(a,1/)') 'MM5: TKE SCALING ABL SCHEME USED'
      endif


* CALCULATE VERTICAL DISCRETIZATION OF ECMWF MODEL
* PARAMETER akm,bkm DESCRIBE THE HYBRID "ETA" COORDINATE SYSTEM

      numskip=nlev_ec-nuvz  ! number of ecmwf model layers not used
                            ! by trajectory model
!      do 40 i=1,nwz
!        j=numskip+i
!        k=nlev_ec+1+numskip+i
!        akm(nwz-i+1)=zsec2(10+j)
!40      bkm(nwz-i+1)=zsec2(10+k)

C     PSK
C     Create artificial model half level with sigmahalf(1)=1., i.e ground level
      do 90 i=nuvz+1,2,-1
         sigmahalf(i)=sigmahalf(i-1)
 90   continue
CPSK Why must sigmahalf(1) be set to 1 ?!
      sigmahalf(1) = 1.0
      nuvz=nuvz + 1
CPSK
C     Create artificial model level for FLEXPART as an composite of half and
C     full sigma levels
      
C NOTE: In FLEXPART versions up to 4.0, the number of model levels was doubled
C upon the transformation to z levels. In order to save computer memory, this is
C not done anymore in the standard version. However, this option can still be
C switched on by replacing the following lines with those below, that are
C currently commented out. For this, similar changes are necessary in
C verttransform.f and verttranform_nests.f
********************************************************************************
CPSK FIXXME Do I need this here ?
C      nz=nuvz
C      if (nz.gt.nzmax) stop 'nzmax too small'
C      do 100 i=1,nuvz
C        aknew(i)=akz(i)
C100     bknew(i)=bkz(i)
CPSK
C Switch on following lines to use doubled vertical resolution
**************************************************************
      nz=nuvz+nwz-1
      if (nz.gt.nzmax) stop 'nzmax (includepar) too small'

CPSK FIXXME BUG ? nwz-1||nwz ?
      do 100 i=1,nwz-1
         sigmanew(2*(i-1)+1)=sigmafull(i)
 100  continue
      do 110 i=2,nuvz
         sigmanew(2*(i-1))=sigmahalf(i)
 110  continue
C End doubled vertical resolution

C Determine the uppermost level for which the convection scheme shall be applied
C by assuming that there is no convection above 50 hPa (for standard SLP)
********************************************************************************
CPSK FIXXME Do I need this ? Yes I do in convmix !!
      do 95 i=1,nuvz-2
C        pint=akz(i)+bkz(i)*101325.
         do 97 jy=0,ny-1
            do 97 ix=0,nxfield-1
               pint=sigmahalf(i)*(ps(ix,jy,1,1)-ptop)+ptop
 97      continue
        if (pint.lt.5000.) goto 96
95      continue

96    nconvlev=i
      if (nconvlev.gt.nconvlevmax-1) then
        nconvlev=nconvlevmax-1
        write(*,*) 'Attention, convection only calculated up to ',
!     +  akz(nconvlev)+bkz(nconvlev)*1013.25,' hPa'
     +  pint,' hPa'
      endif
CPSK FIXXME
      return

 995  write(*,*)
      write(*,*) ' ###########################################'//
     &           '###### '
      write(*,*) '       TRAJECTORY MODEL SUBROUTINE GRIDCHECK:'
      write(*,*) 'AT PRESENT, ONLY LAMBERT CONFORMAL PROJECTION'
      write(*,*) 'IS SUPPORTED.'
      write(*,*) ' ###########################################'//
     &     '###### '

 996  write(*,*)
      write(*,*) ' ###########################################'//
     &           '###### '
      write(*,*) '       TRAJECTORY MODEL SUBROUTINE GRIDCHECK:'
      write(*,*) 'GRID POINTS X ARE NOT THE SAME, PLEASE CHECK '
      write(*,*) 'GRID.TXT AND/OR MM5-INPUTFILES'
      write(*,*) ' ###########################################'//
     &     '###### '

 997  write(*,*)
      write(*,*) ' ###########################################'//
     &           '###### '
      write(*,*) '       TRAJECTORY MODEL SUBROUTINE GRIDCHECK:'
      write(*,*) 'GRID POINTS Y ARE NOT THE SAME, PLEASE CHECK '
      write(*,*) 'GRID.TXT AND/OR MM5-INPUTFILES'
      write(*,*) ' ###########################################'//
     &     '###### '

999   write(*,*)  
      write(*,*) ' ###########################################'//
     &           '###### '
      write(*,*) '       TRAJECTORY MODEL SUBROUTINE GRIDCHECK:'
      write(*,*) ' CAN NOT OPEN INPUT DATA FILE '//wfname(ifn)
      write(*,*) ' ###########################################'//
     &           '###### '
      write(*,*)
      write(*,'(a)') '!!! PLEASE INSERT A NEW CD-ROM AND   !!!'
      write(*,'(a)') '!!! PRESS ANY KEY TO CONTINUE...     !!!'
      write(*,'(a)') '!!! ...OR TERMINATE FLEXTRA PRESSING !!!'
      write(*,'(a)') '!!! THE "X" KEY...                   !!!'
      write(*,*)
      read(*,'(a)') opt
      if(opt.eq.'X') then
        stop
      else
        goto 5
      endif

      end
