      subroutine nn1interpol(yy,nxmax,nymax,nzmax,nx,ny,memind,
C                            i    i     i     i   i  i    i
     +xt,yt,level,itime1,itime2,itime,yint)
C     i  i    i     i      i      i    o
*****************************************************************************
*                                                                           *
*  Interpolation of meteorological fields on 2-d model layers.              *
*  In horizontal nearest neighbour-interpolation is used.                   *
*  Temporally a linear interpolation is used.                               *
*                                                                           *
*  1 first time                                                             *
*  2 second time                                                            *
*                                                                           *
*                                                                           *
*     Author: G. Wotawa                                                     *
*                                                                           *
*     1999-08-11     (Modification of a routine provided by A. Stohl)       *
*                                                                           *
*****************************************************************************
*                                                                           *
* Variables:                                                                *
*                                                                           *
* dt1,dt2              time differences between fields and current position *
* dz1,dz2              z distance between levels and current position       *
* height(nzmax)        heights of the model levels                          *
* indexh               help variable                                        *
* indz                 the level closest to the current trajectory position *
* indzh                help variable                                        *
* itime                current time                                         *
* itime1               time of the first wind field                         *
* itime2               time of the second wind field                        *
* ix,jy                x,y coordinates of lower left subgrid point          *
* level                level at which interpolation shall be done           *
* memind(3)            points to the places of the wind fields              *
* nx,ny                actual field dimensions in x,y and z direction       *
* nxmax,nymax,nzmax    maximum field dimensions in x,y and z direction      *
* xt                   current x coordinate                                 *
* yint                 the final interpolated value                         *
* yt                   current y coordinate                                 *
* yy(0:nxmax,0:nymax,nzmax,3) meteorological field used for interpolation   *
* zt                   current z coordinate                                 *
*                                                                           *
*****************************************************************************

      implicit none

      integer nx,ny,nxmax,nymax,nzmax,memind(2),m,ix,jy
      integer itime,itime1,itime2,level,indexh
      real yy(0:nxmax-1,0:nymax-1,nzmax,2)
      real dt1,dt2,dt,y(2)
      real xt,yt,xt1,yt1,yint



C If point at border of grid -> small displacement into grid
************************************************************
      xt1 = xt
	yt1 = yt
      if (xt.ge.float(nx-1)) xt1=float(nx-1)-0.00001
      if (yt.ge.float(ny-1)) yt1=float(ny-1)-0.00001



***********************************************************************
C 1.) Bilinear horizontal interpolation
C This has to be done separately for 2 fields (Temporal)
********************************************************

C Determine the grid cell the trajectory is within
************************************************** 

      ix=int(xt1)
      jy=int(yt1)

C Loop over 2 time steps
************************
 
      do 10 m=1,2
        indexh=memind(m)
10      y(m)=yy(ix,jy,level,indexh)


*************************************
C 2.) Temporal interpolation (linear)
*************************************

      dt1=float(itime-itime1)
      dt2=float(itime2-itime)
      dt=dt1+dt2

      yint=(y(1)*dt2+y(2)*dt1)/dt
c      if (yint .ne. 0.) print*,yint,'nn'

      return
      end
