      subroutine readoutgrid_nest()
	
********************************************************************************
* $Id: readoutgrid_nest.f,v 1.1 2006/06/21 12:14:37 paul Exp paul $
* $Log: readoutgrid_nest.f,v $
* Revision 1.1  2006/06/21 12:14:37  paul
* Initial revision
*
*                                                                              *
*     This routine reads the user specifications for the output nest.          *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     4 June 1996                                                              *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* dxoutn,dyoutn        grid distances of output nest                           *
* numxgridn,numygridn,numzgrid    nest dimensions                              *
* outlon0n,outlat0n    lower left corner of nest                               *
* outheight(maxzgrid)  height levels of output grid [m]                        *
*                                                                              *
* Constants:                                                                   *
* unitoutgrid          unit connected to file OUTGRID                          *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer i,j
      real xr,xr1,yr,yr1,eps
      parameter(eps=1.e-4)


      if (lc_or_lamphi) stop 'LC OUTGRID implementation not yet ready!'

C Open the OUTGRID file and read output grid specifications
***********************************************************

      open(unitoutgrid,file=path(1)(1:len(1))//'OUTGRID_NEST',
     +status='old',err=999)

      call skplin(5,unitoutgrid)

C 1.  Read horizontal grid specifications
*****************************************

      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f11.4)') outlon0n
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f11.4)') outlat0n
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,i5)') numxgridn
      if (numxgridn.gt.maxxgridn) stop 'Please increase maxxgridn'
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,i5)') numygridn
      if (numygridn.gt.maxygridn) stop 'Please increase maxygridn'
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f12.5)') dxoutn
      if (lc_or_lamphi) dxoutn=dxoutn*1000.
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f12.5)') dyoutn
      if (lc_or_lamphi) dyoutn=dyoutn*1000.
!      if ( dyoutn.ne.dxoutn) goto 995


C Check validity of output grid (shall be within model domain)
**************************************************************
      if (lc_or_lamphi) then
        print '("OUTGRID NEST is using Lambert-Conformal projection.")'
	else  
        print '("OUTGRID NEST is using a regular lat/lon grid.")'
	endif  

      call ij_to_latlon(float(nxfield-1),float(ny-1),yr1,xr1)

      if (lc_or_lamphi) then
CPS not implemented, ij-xy transformation not checked	
        call map_set(code_sav,outlat0n,outlon0n,1.,1.,dxoutn,stdlon_sav,
     &               truelat1_sav,truelat2_sav)
        call ij_to_latlon(float(numxgridn),float(numygridn),yr,xr)
      
        do 992 j=1,numygridn
         do 992 i=1,numxgridn
          outlonn(i,j)=-999.
          outlatn(i,j)=-999.
992     continue

        do 991 j=1,numygridn
         do 991 i=1,numxgridn
          call ij_to_latlon(float(i),float(j),yr,xr) 
          outlonn(i-1,j-1)=xr
          outlatn(i-1,j-1)=yr
991     continue


      else ! lat-lon output grid
	
        xr=outlon0n+float(numxgridn)*dxoutn
        yr=outlat0n+float(numygridn)*dyoutn
        outlat1n_sav=outlat0n
        outlon1n_sav=outlon0n
        print '("Outgrid Nest:",1/,"  Longitude range: ",f10.1," ",
     &    f10.1,"   Grid distance (deg): ",f10.4)',outlon0n,xr,dxoutn
            
        print '("  Latitude range:  ",f10.1," ",f10.1,
     &     "   Grid distance (deg): ",f10.4,1/)',outlat0n,yr,dyoutn

      endif

C Set Map back to MM5 input files
      call map_set(code_sav,lat1_sav,lon1_sav,1.,1.,dx_sav,stdlon_sav,
     &     truelat1_sav,truelat2_sav)
!      xr=outlon0n+float(numxgridn)*dxoutn
!      yr=outlat0n+float(numygridn)*dyoutn
!      xr1=xlon0+float(nxmin1)*dx
!      yr1=ylat0+float(nymin1)*dy

      if ( outlon0n+eps .lt. xlon0 .or. outlat0n+eps .lt. ylat0
     &  .or. xr .gt. xr1+eps .or. yr .gt. yr1+eps) then
        write(*,*) ' #### FLEXPART MODEL ERROR! PART OF OUTPUT    ####'
        write(*,*) ' #### NEST IS OUTSIDE MODEL DOMAIN. CHANGE    ####'
        write(*,*) ' #### FILE OUTGRID IN DIRECTORY               ####'
        write(*,'(a)') path(1)(1:len(1))
        stop
      endif
      if (numxgridn .gt. maxxgridn .or. numygridn .gt. maxygridn) then
        write(*,*) ' #### FLEXPART MODEL ERROR! DIMENSIONS OF     ####'
        write(*,*) ' #### OUTPUT NEST EXCEED MAXIMUM VALUES.      ####'
        write(*,*) ' #### CHANGE FILE $FLEXPART/options/OUTGRID.  ####'
        stop
      endif

      xoutshiftn=xlon0-outlon0n
      youtshiftn=ylat0-outlat0n
      close(unitoutgrid)
      return

 995  write(*,*) ' #### FLEXPART MODEL ERROR! GRIDRESOLUTION IN #### '
      write(*,*) ' #### FILE "OUTGRID" MUST BE THE SAME.        #### '
      stop

999   write(*,*) ' #### FLEXPART MODEL ERROR! FILE OUTGRID_NEST #### '
      write(*,*) ' #### CANNOT BE OPENED IN THE DIRECTORY       #### '
      write(*,*) ' #### xxx/flexpart/options                    #### '
      stop

      end
