      subroutine readwind(indj,n,uuh,vvh,wwh)
***********************************************************************
* $Id: readwind.f,v 1.1 2006/08/09 10:03:29 paul Exp paul $
* $Log: readwind.f,v $
* Revision 1.1  2006/08/09 10:03:29  paul
* Initial revision
*
*                                                                     * 
*             TRAJECTORY MODEL SUBROUTINE READWIND                    *
*                                                                     *
***********************************************************************
*                                                                     * 
*             AUTHOR:      G. WOTAWA                                  *
*             DATE:        1997-08-05                                 *
*             LAST UPDATE: 2000-10-17, Andreas Stohl                  *
*                                                                     * 
***********************************************************************
*  Changes, Bernd C. Krueger, Feb. 2001:
*        Variables tth and qvh (on eta coordinates) in common block
***********************************************************************
*                                                                     *
* DESCRIPTION:                                                        *
*                                                                     *
* READING OF ECMWF METEOROLOGICAL FIELDS FROM INPUT DATA FILES. THE   *
* INPUT DATA FILES ARE EXPECTED TO BE AVAILABLE IN GRIB CODE          *
*                                                                     *
* INPUT:                                                              *
* indj               indicates number of the wind field to be read in *
* n                  temporal index for meteorological fields (1 to 3)*
*                                                                     *
* IMPORTANT VARIABLES FROM COMMON BLOCK:                              *
*                                                                     *
* wfname             File name of data to be read in                  *
* nx,ny,nuvz,nwz     expected field dimensions                        *
* nlev_ec            number of vertical levels ecmwf model            *
* uu,vv,ww           wind fields                                      *
* tt,qv              temperature and specific humidity                *
* ps                 surface pressure                                 *
*                                                                     *
***********************************************************************
*

      include 'includepar'
      include 'includecom'

      real uuh(0:nxmax-1,0:nymax-1,nuvzmax)
      real vvh(0:nxmax-1,0:nymax-1,nuvzmax)
      real wwh(0:nxmax-1,0:nymax-1,nuvzmax) !nwzmax ?
      real clwprof(nuvzmax),ttprof(nuvzmax)
      real cloudtop,cloudwater
      integer indj,i,j,k,n,levdiff2,ifield,iumax,iwmax,lunit
!      real tkeh(0:nxmax-1,0:nymax-1,nuvzmax) !INTO COMMON

* VARIABLES AND ARRAYS NEEDED FOR GRIB DECODING

C dimension of isec2 at least (22+n), where n is the number of parallels or
C meridians in a quasi-regular (reduced) Gaussian or lat/long grid

C dimension of zsec2 at least (10+nn), where nn is the number of vertical
C coordinate parameters

      integer isec0(2),isec1(56),isec2(22+nxmax+nymax),isec3(2)
      integer isec4(64),inbuff(jpack),ilen,iswap,ierr,iword
      real zsec2(60+2*nuvzmax),zsec3(2),zsec4(jpunp)
      real xaux,yaux,xaux0,yaux0,eps
      parameter(eps=1.e-4)
      real ewss(0:nxmax-1,0:nymax-1),nsss(0:nxmax-1,0:nymax-1)
      real plev1,pmean,tv,fu,hlev1,ff10m,fflev1

      character*1 yoper
      logical hflswitch,strswitch

      data yoper/'D'/

      hflswitch=.false.
      strswitch=.false.
      levdiff2=nlev_ec-nwz+1
      iumax=0
      iwmax=0

*
* OPENING OF DATA FILE (GRIB CODE)
*
5     call pbopen(lunit,path(3)(1:len(3))//wfname(indj),'r',ierr)
      if(ierr.lt.0) goto 999

      ifield=0
10    ifield=ifield+1
*
* GET NEXT FIELDS
*
      call pbgrib(lunit,inbuff,jpack,ilen,ierr)
      if(ierr.eq.-1) goto 50    ! EOF DETECTED
      if(ierr.lt.-1) goto 888   ! ERROR DETECTED

      ierr=1

C Check whether we are on a little endian or on a big endian computer
*********************************************************************

c     if (inbuff(1).eq.1112101447) then         ! little endian, swap bytes
c       iswap=1+ilen/4
c       call swap32(inbuff,iswap)
c     else if (inbuff(1).ne.1196575042) then    ! big endian
c       stop 'subroutine gridcheck: corrupt GRIB data'
c     endif


      call gribex(isec0,isec1,isec2,zsec2,isec3,zsec3,isec4,
     &            zsec4,jpunp,inbuff,jpack,iword,yoper,ierr)
      if (ierr.ne.0) goto 888   ! ERROR DETECTED

      if(ifield.eq.1) then

* CHECK GRID SPECIFICATIONS

        if(isec2(2).ne.nxfield) stop 'READWIND: NX NOT CONSISTENT'
        if(isec2(3).ne.ny) stop 'READWIND: NY NOT CONSISTENT'
        if(isec2(12).ne.nlev_ec)
     &    stop 'READWIND: VERTICAL DISCRETIZATION NOT CONSISTENT'

CPS in this version xaux, yaux is not used
CPS but I have subtracted 1 from the grid indices for compatibility
CPS        call ij_to_latlon(float(0),float(ny-1),yaux,xaux)
CPS        call ij_to_latlon(float(nxfield-1),float(0),yaux,xaux)
!        xaux=float(isec2(5))/1000.!+float(nxshift)*dx
!        yaux=float(isec2(7))/1000.

        xaux0=xlon0
        yaux0=ylat0

CPS        if (xaux.lt.0.) xaux=xaux+360.
CPS        if (yaux.lt.0.) yaux=yaux+360.
        if (xaux0.lt.0.) xaux0=xaux0+360.
        if (yaux0.lt.0.) yaux0=yaux0+360.
!        if(abs(xaux-xaux0).gt.eps) 
!     &    stop 'READWIND: LOWER LEFT LONGITUDE NOT CONSISTENT'
!        if(abs(yaux-yaux0).gt.eps)
!     &    stop 'READWIND: LOWER LEFT LATITUDE NOT CONSISTENT'
      endif

      do 20 j=0,nymin1
        do 20 i=0,nxfield-1
          k=isec1(8)

c	if (zsec4(nxfield*(ny-j-1)+i+1) .lt. -1000. .or.
c     & zsec4(nxfield*(ny-j-1)+i+1) .gt. 1.e6) then
c       print*,'rdwind',isec1(6),i,j,k,zsec4(nxfield*(ny-j-1)+i+1)
c	endif 

          if(isec1(6).eq.130) tth(i,j,nlev_ec-k+1,n)=   !! TEMPERATURE
     &                        zsec4(nxfield*(ny-j-1)+i+1)

          if(isec1(6).eq.131) uuh(i,j,nlev_ec-k+1)= !! U VELOCITY
     &                        zsec4(nxfield*(ny-j-1)+i+1)
          
          if(isec1(6).eq.132) vvh(i,j,nlev_ec-k+1)=     !! V VELOCITY
     &                        zsec4(nxfield*(ny-j-1)+i+1)

          if(isec1(6).eq.133) then                      !! SPEC. HUMIDITY  
            qvh(i,j,nlev_ec-k+2,n)=zsec4(nxfield*(ny-j-1)+i+1)
            if (qvh(i,j,nlev_ec-k+2,n) .lt. 0.)
     &          qvh(i,j,nlev_ec-k+2,n) = 0.
c               this is necessary because the gridded data may contain
c               spurious negative values
          endif

          if(isec1(6).eq.134) ps(i,j,1,n)=              !! REF. SURF. PRESS.
     &                        zsec4(nxfield*(ny-j-1)+i+1)

          if(isec1(6).eq.135)  wwh(i,j,nlev_ec-k+2)= !! W VELOCITY
     &                         zsec4(nxfield*(ny-j-1)+i+1)
          if(isec1(6).eq.135.and.k.eq.1) wwh(i,j,nlev_ec+1)=
     &                        zsec4(nxfield*(ny-j-1)+i+1)!!W VELOICTY AT PTOP
          
          if(isec1(6).eq.201) tkeh(i,j,nlev_ec-k+1)=     !! TKE
     &                        max(zsec4(nxfield*(ny-j-1)+i+1),1.e-4)
          
          if(isec1(6).eq.246) clw(i,j,nlev_ec-k+1,n)= !! CLOUD WATER
     &                        zsec4(nxfield*(ny-j-1)+i+1)

          if(isec1(6).eq.202) sd(i,j,1,n)=              !! SNOW COVER
     &                        zsec4(nxfield*(ny-j-1)+i+1)

          if(isec1(6).eq.142) then                      !! LARGE SCALE PREC.
            lsprec(i,j,1,n)=zsec4(nxfield*(ny-j-1)+i+1)
            if (lsprec(i,j,1,n).lt.0.) lsprec(i,j,1,n)=0.
          endif

          if(isec1(6).eq.143) then                      !! CONVECTIVE PREC.
            convprec(i,j,1,n)=zsec4(nxfield*(ny-j-1)+i+1)
            if (convprec(i,j,1,n).lt.0.) convprec(i,j,1,n)=0.
          endif

          if(isec1(6).eq.146) sshf(i,j,1,n)=            !! SENS. HEAT FLUX
     &                        -1.*zsec4(nxfield*(ny-j-1)+i+1)
          if((isec1(6).eq.146).and.(zsec4(nxfield*(ny-j-1)+i+1).ne.0.))
     &    hflswitch=.true.    ! Heat flux available

          if(isec1(6).eq.176) then                      !! SOLAR RADIATION
            ssr(i,j,1,n)=zsec4(nxfield*(ny-j-1)+i+1)
            if (ssr(i,j,1,n).lt.0.) ssr(i,j,1,n)=0.
          endif

          if(isec1(6).eq.180) ustar(i,j,1,n)=           !! USTAR
     &                        zsec4(nxfield*(ny-j-1)+i+1)

          if(isec1(6).eq.159) hmix(i,j,1,n)=            !! HMIX
     &                        zsec4(nxfield*(ny-j-1)+i+1)

C          if(((isec1(6).eq.180).or.(isec1(6).eq.181)).and.
C     &    (zsec4(nxfield*(ny-j-1)+i+1).ne.0.)) strswitch=.true.    ! stress available

          if(isec1(6).eq.129) oro(i,j)=                 !! ECMWF OROGRAPHY
     &                        zsec4(nxfield*(ny-j-1)+i+1)

          if(isec1(6).eq.172) then                      !! LAND SEA MASK
             if (int(zsec4(nxfield*(ny-j-1)+i+1)).eq.16) then
                lsm(i,j)=0.      
             else
                lsm(i,j)=1.
             endif
          endif
          if(isec1(6).eq.131) iumax=max(iumax,nlev_ec-k+1)
          if(isec1(6).eq.135) iwmax=max(iwmax,nlev_ec-k+2)
20    continue

      goto 10                      !! READ NEXT LEVEL OR PARAMETER
*
* CLOSING OF INPUT DATA FILE
*
50    call pbclose(lunit,ierr)     !! FINNISHED READING / CLOSING GRIB FILE

      if(levdiff2.eq.0) then
        iwmax=nlev_ec+1
C        do 60 i=0,nxmin1
C          do 60 j=0,nymin1
C60          wwh(i,j,nlev_ec+1)=0.
      endif


C For global fields, assign the leftmost data column also to the rightmost
C data column; if required, shift whole grid by nxshift grid points
**************************************************************************

C      if (xglobal) then
C        call shift_field_0(ewss,nxfield,ny)
C        call shift_field_0(nsss,nxfield,ny)
C        call shift_field_0(oro,nxfield,ny)
C        call shift_field_0(excessoro,nxfield,ny)
C        call shift_field_0(lsm,nxfield,ny)
C        call shift_field(ps,nxfield,ny,1,1,2,n)
C        call shift_field(sd,nxfield,ny,1,1,2,n)
C        call shift_field(msl,nxfield,ny,1,1,2,n)
C        call shift_field(tcc,nxfield,ny,1,1,2,n)
C        call shift_field(u10,nxfield,ny,1,1,2,n)
C        call shift_field(v10,nxfield,ny,1,1,2,n)
C        call shift_field(tt2,nxfield,ny,1,1,2,n)
C        call shift_field(td2,nxfield,ny,1,1,2,n)
C        call shift_field(lsprec,nxfield,ny,1,1,2,n)
C        call shift_field(convprec,nxfield,ny,1,1,2,n)
C        call shift_field(sshf,nxfield,ny,1,1,2,n)
C        call shift_field(ssr,nxfield,ny,1,1,2,n)
C        call shift_field(tth,nxfield,ny,nuvzmax,nuvz,2,n)
C        call shift_field(qvh,nxfield,ny,nuvzmax,nuvz,2,n)
C        call shift_field(uuh,nxfield,ny,nuvzmax,nuvz,1,1)
C        call shift_field(vvh,nxfield,ny,nuvzmax,nuvz,1,1)
C        call shift_field(wwh,nxfield,ny,nwzmax,nwz,1,1)
C      endif

C      do 65 i=0,nxmin1
C        do 65 j=0,nymin1
C65        surfstr(i,j,1,n)=sqrt(ewss(i,j)**2+nsss(i,j)**2)

C      if ((.not.hflswitch).or.(.not.strswitch)) then
C        write(*,*) 'WARNING: No flux data contained in GRIB file ',
C     +  wfname(indj)
 
C CALCULATE USTAR AND SSHF USING THE PROFILE METHOD
C As ECMWF has increased the model resolution, such that now the first model
C level is at about 10 m (where 10-m wind is given), use the 2nd ECMWF level
C (3rd model level in FLEXPART) for the profile method 
****************************************************************************

C        do 1500 i=0,nxmin1
C          do 1500 j=0,nymin1
C            plev1=akz(3)+bkz(3)*ps(i,j,1,n)
C            pmean=0.5*(ps(i,j,1,n)+plev1)
C            tv=tth(i,j,3,n)*(1.+0.61*qvh(i,j,3,n))
C            fu=-r_air*tv/ga/pmean
C            hlev1=fu*(plev1-ps(i,j,1,n))   ! HEIGTH OF FIRST MODEL LAYER
C            ff10m= sqrt(u10(i,j,1,n)**2+v10(i,j,1,n)**2)
C            fflev1=sqrt(uuh(i,j,3)**2+vvh(i,j,3)**2)
C            call pbl_profile(ps(i,j,1,n),td2(i,j,1,n),hlev1,
C     &                       tt2(i,j,1,n),tth(i,j,3,n),ff10m,fflev1,
C     &                       surfstr(i,j,1,n),sshf(i,j,1,n))
C            if(sshf(i,j,1,n).gt.200.) sshf(i,j,1,n)=200.
C            if(sshf(i,j,1,n).lt.-400.) sshf(i,j,1,n)=-400.
C1500        continue
C      endif

         do 70 i=0,nxmin1
            do 70 j=0,nymin1

C CALCULATE TOTAL INTEGRATED CLOUD WATER AND CLOUD TOP HEIGHT
************************************************************************
               do 61 k=1,nuvz-1
                  ttprof(k)  = tth(i,j,k+1,n)
                  clwprof(k) = clw(i,j,k,n)
  61            continue
               cltop(i,j,1,n)=cloudtop(ps(i,j,1,n),clwprof,
     &              nuvz-1,sigmafull,ptop,p00,ts0,ta)
               ticlw(i,j,1,n)=cloudwater(ps(i,j,1,n),ttprof,clwprof,
     &              nuvz-1,sigmahalf,sigmafull,ptop,p00,ts0,ta)

C SET SURFACE VALUES TO VALUES OF 1ST MODEL LEVEL
************************************************************************

               uuh(i,j,1)=uuh(i,j,2)
               vvh(i,j,1)=vvh(i,j,2)
               qvh(i,j,1,n)=qvh(i,j,2,n)
               tth(i,j,1,n)=tth(i,j,2,n)
               tkeh(i,j,1) =tkeh(i,j,2)
 70      continue

CPSK FIXXME nuvz-1 ?!?
      if(iumax.ne.nuvz-1) stop 'READWIND: NUVZ NOT CONSISTENT'
      if(iwmax.ne.nwz)    stop 'READWIND: NWZ NOT CONSISTENT'

      return    
888   write(*,*) ' #### FLEXPART MODEL ERROR! WINDFIELD         #### ' 
      write(*,*) ' #### ',wfname(indj),'                    #### '
      write(*,*) ' #### IS NOT GRIB FORMAT !!!                  #### '
      stop 'Execution terminated'
999   write(*,*) ' #### FLEXPART MODEL ERROR! WINDFIELD         #### ' 
      write(*,*) ' #### ',wfname(indj),'                    #### '
      write(*,*) ' #### CANNOT BE OPENED !!!                    #### '
      stop 'Execution terminated'

      end
